import type { RelationStruct } from "ts-common";
import { getNodeCypher } from "./node";
import type { QueryCacheData } from "./model";

export const NoNodeError = new Error("relation must have a node");

export function getRelationCypher(JSONQuery: RelationStruct): [string | undefined, QueryCacheData] {
    let label = "";
    if (JSONQuery.label) {
        label = `:${JSONQuery.label}`;
    }
    let id = "";
    if (JSONQuery.id) {
        id = JSONQuery.id;
    }
    let depth = "";
    if (JSONQuery.depth) {
        depth = `*${JSONQuery.depth.min}..${JSONQuery.depth.max}`;
    }

    const cacheData: QueryCacheData = { entities: [], relations: [] };

    if (id) {
        cacheData.relations.push({ id, queryId: "" });
    }

    let cypher = `[${id}${label}${depth}]`;

    if (!JSONQuery.node) {
        return [undefined, cacheData];
    }

    const [nodeCypher, subCache] = getNodeCypher(JSONQuery.node);

    cacheData.entities.push(...subCache.entities);
    cacheData.relations.push(...subCache.relations);
    if (JSONQuery.direction === "TO") {
        cypher += `->${nodeCypher}`;
    } else if (JSONQuery.direction === "FROM") {
        cypher += `-${nodeCypher}`;
    } else {
        cypher += `-${nodeCypher}`;
    }
    return [cypher, cacheData];
}